import logging
import hashlib
from xml.etree import ElementTree as ET

from django.utils.translation import ugettext_lazy as _
from django.conf import settings
from django.core.cache import cache
from django.template import Template

import feedparser
import requests
from requests.exceptions import RequestException

from cms.plugin_base import CMSPluginBase
from cms.plugin_pool import plugin_pool
from models import DataSourcePlugin as DataSourcePluginModel, DataSource, RawInlineTemplatePlugin as RawInlineTemplatePluginModel
import signature


logger = logging.getLogger(__name__)

CACHE_SENTINEL = object()

class Data(object):
    '''Encapsulate data from a source'''
    MAPPING = {
            DataSource.JSON: 'json',
            DataSource.RSS: 'rss',
            DataSource.HTML: 'html',
            DataSource.XML: 'xml',
    }

    def __init__(self, data_source, context, limit, refresh):
        self.data_source = data_source
        self.kind = self.MAPPING.get(data_source.mime_type)
        self.context = context
        self.url = Template(self.data_source.url).render(self.context)
        self.limit = limit
        self.refresh = refresh
        self.key = hashlib.md5('datasource-{self.data_source.id}-{self.url}-{self.limit}-{self.refresh}'.format(self=self)).hexdigest()

    def get_content(self):
        content = cache.get(self.key, CACHE_SENTINEL)
        if content is not CACHE_SENTINEL:
            return content
        if self.kind is not None:
            try:
                url = self.url
                if self.data_source.signature_key:
                    url = signature.sign_url(url.encode('ascii'),
                            self.data_source.signature_key.encode('utf-8'),
                            algo=self.data_source.hash_algo)
                logger.debug('getting data source %r from url %r', self.data_source.name, url)
                request = requests.get(url)
            except RequestException:
                logger.exception('unable to retrieve %r', unicode(self.data_source))
                content = None
            else:
                content = getattr(self, 'get_content_'+self.kind)(request)
            cache.set(self.key, content, self.refresh)
            return content
        return NotImplemented
    content = property(get_content)

    def get_content_json(self, request):
        try:
            return request.json()
        except ValueError:
            return None

    def get_content_rss(self, request):
        result = feedparser.parse(request.content)
        result.entries = sorted(result.entries, key=lambda e: e['updated_parsed'])[:self.limit]
        return result

    def get_content_html(self, request):
        return request.text

    def get_content_xml(self, request):
        return ET.fromstring(request.content)

class RawInlineTemplatePlugin(CMSPluginBase):
    model = RawInlineTemplatePluginModel
    name = _('Raw Inline Template Plugin')
    render_template = "data_source_plugin/plugin.html"
    text_enabled = True

    def icon_src(self, instance):
        return settings.STATIC_URL + u"cms/images/plugins/link.png"

class DataSourcePlugin(CMSPluginBase):
    model = DataSourcePluginModel
    name = _('Data Source Plugin')
    render_template = None
    text_enabled = True

    def render(self, context, instance, placeholder):
        context['data_sources'] = []
        for source in instance.sources.all():
            context['data_sources'].append(
                    Data(source.source, context, instance.limit,
                        instance.refresh))
        return context

plugin_pool.register_plugin(RawInlineTemplatePlugin)
plugin_pool.register_plugin(DataSourcePlugin)
